/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "drawui.h"
#include "ui/uitextbox.h"
#include "util/memory.h"
#include "assert/assert.h"

void drawUIInit(void) {
}

void drawUI() {
  drawUITextbox();
}

void drawUITextbox() {
  if(!UI_TEXTBOX.visible) return;

  uitextbox_t *textbox = &UI_TEXTBOX;

   // Semi-transparent dark blue
  Color background = (Color){ 0, 0, 128, 128 };
  DrawRectangle(
    0, RENDER_HEIGHT - UI_TEXTBOX_HEIGHT,
    UI_TEXTBOX_WIDTH, UI_TEXTBOX_HEIGHT,
    background
  );

  BeginBlendMode(BLEND_ALPHA);

  if(UI_TEXTBOX.charsRevealed > 0) {
    char_t buffer[UI_TEXTBOX_CHARS_PER_LINE + 1];// +1 for null term
    uint8_t charsRendered = 0;
    uitextbox_t *textbox = &UI_TEXTBOX;

    // For each line
    for(uint8_t i = 0; i < UI_TEXTBOX_LINES_PER_PAGE; i++) {
      // Get count of chars in the line
      uint8_t lineLength = UI_TEXTBOX.lineLengths[
        i + (UI_TEXTBOX.page * UI_TEXTBOX_LINES_PER_PAGE)
      ];
      if(lineLength == 0) continue;

      // Determine how many chars left to render
      uint8_t lineChars = UI_TEXTBOX.charsRevealed - charsRendered;
      
      // Don't render more than in line
      if(lineChars > lineLength) lineChars = lineLength;
      assertTrue(lineChars > 0, "Line chars must be greater than 0");

      // Update how many rendered
      charsRendered += lineChars;

      // Copy string from VN Textbox...
      memoryCopy(
        buffer,
        UI_TEXTBOX.text + (UI_TEXTBOX.page * UI_TEXTBOX_CHARS_PER_PAGE) +
        (i * UI_TEXTBOX_CHARS_PER_LINE),
        lineChars
      );
      // Null term string
      buffer[lineChars] = '\0'; // Null-terminate the string

      // Render the text
      DrawText(
        buffer,
        UI_TEXTBOX_PADDING_X + UI_TEXTBOX_BORDER_WIDTH,
        (
          (RENDER_HEIGHT - UI_TEXTBOX_HEIGHT) +
          UI_TEXTBOX_PADDING_Y + UI_TEXTBOX_BORDER_HEIGHT +
          (i * FONT_LINE_HEIGHT)
        ),
        FONT_HEIGHT,
        WHITE
      );

      // Check if we're done rendering text
      if(UI_TEXTBOX.charsRevealed - charsRendered == 0) break;
    }
  }

  EndBlendMode();
}